 /*---[ firestarter.c ]-----------------------------------------------
 * Copyright (C) 2000 Tomas Junnonen (majix@sci.fi)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * The main application file
 *--------------------------------------------------------------------*/

#include <status-docklet.h>

#include <errno.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>

#include "firestarter.h"
#include "menus.h"
#include "parse.h"
#include "druid.h"
#include "service.h"
#include "preferences.h"
#include "modrules.h"

#include "grey1.xpm"
#include "blue1.xpm"
#include "red1.xpm"

#include <libgnomeui/gnome-window-icon.h>

GtkWidget *window;
GtkWidget *vbox;
GtkWidget *panelpic;

gboolean logenabled = TRUE;
gboolean running = FALSE;

void set_panel_pic (gchar *pixmap[]);

/*---[ appbar_update ]------------------------------------------------
 * Updates the application status bar with the state of the firewall
 *--------------------------------------------------------------------*/
void
appbar_update (void)
{
	if (running)
		gnome_appbar_set_default (GNOME_APPBAR (appbar),
					 _("Firewall running"));
	else
		gnome_appbar_set_default (GNOME_APPBAR (appbar),
					_("Firewall stopped"));
}

/*---[ clist_row_select ]---------------------------------------------
 * Attach current line data to a clist. uggly
 *--------------------------------------------------------------------*/
void
clist_row_select (GtkWidget *clist, gint row, gint column,
		  GdkEventButton *event, gpointer data)
{
	gint *clistrow = GINT_TO_POINTER (row+1);
	gint *clistcol = GINT_TO_POINTER (column+1);

	gtk_object_set_data (GTK_OBJECT (clist), "row", clistrow);
	gtk_object_set_data (GTK_OBJECT (clist), "col", clistcol);

	if (clist != denyallclist)
		gtk_clist_unselect_all (GTK_CLIST (denyallclist));
	else dynclist = (gpointer *) denyallclist;

	if (clist != allowallclist)
		gtk_clist_unselect_all (GTK_CLIST (allowallclist));
	else dynclist = (gpointer *) allowallclist;

	if (clist != allowsmclist)
		gtk_clist_unselect_all (GTK_CLIST (allowsmclist));
	else dynclist = (gpointer *) allowsmclist;

	if (clist != allowsaclist)
		gtk_clist_unselect_all (GTK_CLIST (allowsaclist));
	else dynclist = (gpointer *) allowsaclist;
}

/*---[ clist_row_unselect ]-------------------------------------------
 * Clear current line data from a clist
 *--------------------------------------------------------------------*/
void
clist_row_unselect (GtkWidget *clist)
{
	gint *clistrow = NULL;
	gint *clistcol = NULL;

	if (clist == (GtkWidget *)dynclist)
		dynclist = NULL;

	gtk_object_set_data (GTK_OBJECT (clist), "row", clistrow);
	gtk_object_set_data (GTK_OBJECT (clist), "col", clistcol);
}

/*---[ hide_window_cb ]-----------------------------------------------
 * Hides the main window, either when "destroyed" or docklet clicked.
 *--------------------------------------------------------------------*/
void
hide_window_cb (GtkWidget *window, gpointer data)
{
	if (GTK_WIDGET_VISIBLE (window))
		gtk_widget_hide_all (window);
	else
		gtk_widget_show_all (window);

	if (running)
		set_panel_pic (blue1_xpm);
}

/*---[ savelog ]------------------------------------------------------
 * Saves the log to a disk file, called from create_savelog_filesel
 *--------------------------------------------------------------------*/
void
savelog (GtkWidget *widget, gpointer data) 
{
	gchar *text;
	gint i = 0;
	FILE *f;
	gchar *savefile;

	savefile = gtk_file_selection_get_filename (GTK_FILE_SELECTION (data));

	f = fopen (savefile, "w");

	while (gtk_clist_get_text (GTK_CLIST (hitlogclist), i, 0, &text)) {
		fprintf (f, "%s\t\t", text);
		
		gtk_clist_get_text (GTK_CLIST (hitlogclist), i, 1, &text);
		fprintf (f, "%s\t\t", text);

		gtk_clist_get_text (GTK_CLIST (hitlogclist), i, 2, &text);
		fprintf (f, "%s\t\t", text);

		gtk_clist_get_text (GTK_CLIST (hitlogclist), i, 3, &text);
		fprintf (f, "%s\n", text);

		i++;
	}
	fclose (f);
}

/*---[ create_savelog_filesel ]---------------------------------------
 * Creates a file selection dialog, passed to savelog
 *--------------------------------------------------------------------*/
void
create_savelog_filesel (GtkWidget *widget, gpointer data)
{
	if (logenabled) {
		GtkWidget *filesel;

		filesel = gtk_file_selection_new _("Select file to save log to");

		gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->ok_button),
				    "clicked", GTK_SIGNAL_FUNC (savelog),
				    (gpointer) filesel);

		gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->ok_button),
					   "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
					   (gpointer) filesel);

		gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->cancel_button),
					   "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
					   (gpointer) filesel);

		gtk_widget_show (filesel);
	}
}

/*---[ stop_firewall ]------------------------------------------------
 * Flushes, zeroes and sets the default policy of the chains
 *--------------------------------------------------------------------*/
void
stop_firewall (void)
{
	if (geteuid ()) {
		GtkWidget *dialog=gnome_error_dialog _("You must be root to manipulate the firewall.");
		gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
		return;
	}

	system ("/sbin/ipchains -X; /sbin/ipchains -F; /sbin/ipchains -Z;");
	system ("/sbin/ipchains -P input ACCEPT; /sbin/ipchains -P forward ACCEPT;"
		"/sbin/ipchains -P output ACCEPT;");
	g_print _("Firewall stopped, network traffic is now flowing freely\n");

	running = FALSE;
	set_panel_pic (grey1_xpm);

	appbar_update ();
}

/*---[ restart_firewall ]---------------------------------------------
 * Executes the firewall script
 *--------------------------------------------------------------------*/
void
restart_firewall (void)
{
	gchar *commandline = g_strjoin (NULL, "/bin/sh ",
					gnome_config_get_string ("/firestarter/Files/scriptfile=/usr/local/firestarter/firewall.sh"),
					NULL);

	if (geteuid ()) {
		GtkWidget *dialog=gnome_error_dialog _("You must be root to manipulate the firewall.");
		gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
		return;
	}

	system (commandline);
	g_print _("Firewall script restarted\n");
	running = TRUE;

	set_panel_pic (blue1_xpm);
	g_free (commandline);
	appbar_update ();
}

/*---[ clear_hitlist ]------------------------------------------------
 * Clears the log CList
 *--------------------------------------------------------------------*/
void
clear_hitlist (gpointer data)
{
	if (running)
		set_panel_pic (blue1_xpm);
	gtk_clist_clear (GTK_CLIST (hitlogclist));
	clist_row_unselect (hitlogclist);
}

/*---[ reload_hitlist ]-----------------------------------------------
 * Reads the entire log file into the CList
 *--------------------------------------------------------------------*/
void
reload_hitlist (gpointer data)
{
	if (logenabled) {
		char buf[512];
		char *logpath = gnome_config_get_string ("/firestarter/Files/logfile=/var/log/messages");
		FILE *f;

		f = fopen (logpath, "r");

		gtk_clist_freeze (GTK_CLIST (hitlogclist));
		gtk_clist_clear (GTK_CLIST (hitlogclist));
		while (fgets (buf, 512, f) != NULL) {
			if (strstr (buf, "DENY"))
				parse_hit_log (buf);
		}
		gtk_clist_thaw (GTK_CLIST (hitlogclist));
		fclose (f);
	}
}

/*---[ poll_log_timeout ]---------------------------------------------
 * Polls the logfile every 700ms for change, if change, parse lines
 *--------------------------------------------------------------------*/
int
poll_log_timeout (gpointer file)
{
	char buf[512];

	if (fgets (buf, 512, file) != NULL) {
		if (strstr (buf, "DENY"))
			if (parse_hit_log (buf))
				set_panel_pic (red1_xpm);
	}
	return TRUE;
}

/*---[ show_about ]---------------------------------------------------
 * Creates the about dialog
 *--------------------------------------------------------------------*/
void
show_about (GtkWidget *widget, gpointer data)
{
	static GtkWidget *dialog = NULL;

	if (dialog != NULL) {
		g_assert (GTK_WIDGET_REALIZED (dialog));
		gdk_window_show (dialog->window);
		gdk_window_raise (dialog->window);
	}
	else {
		const gchar *authors[] = {"Tomas Junnonen <majix@sci.fi>",
					  NULL};

		dialog = gnome_about_new ("Firestarter", VERSION,
					  "(C) 2000 Tomas Junnonen", authors,
					  _("An all-in-one Linux firewall utility for GNOME"),
					  NULL);

		gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
				    GTK_SIGNAL_FUNC (gtk_widget_destroyed), &dialog);

		gtk_widget_show (dialog);
	}
}

/*---[ set_panel_pic ]------------------------------------------------
 * Sets the picture on the docklet to *pixmap
 *--------------------------------------------------------------------*/
void
set_panel_pic (gchar *pixmap[])
{
	gtk_container_remove (GTK_CONTAINER (vbox), panelpic);
	panelpic = gnome_pixmap_new_from_xpm_d (pixmap);
	gtk_box_pack_end (GTK_BOX (vbox), panelpic, FALSE, FALSE, 2);
	gtk_widget_show (panelpic);
}

/*---[ status_docklet_build_plug ]------------------------------------
 * Builds the docklet plug
 *--------------------------------------------------------------------*/
static void
status_docklet_build_plug (StatusDocklet *docklet, GtkWidget *plug)
{
	GtkWidget *eventbox;
	GtkWidget *frame;

	g_return_if_fail (docklet != NULL);
	g_return_if_fail (IS_STATUS_DOCKLET (docklet));

	eventbox = gtk_event_box_new ();
	gtk_widget_set_events (eventbox, GDK_BUTTON_PRESS_MASK);
	gtk_signal_connect_object (GTK_OBJECT (eventbox), "button_press_event",
				   GTK_SIGNAL_FUNC (hide_window_cb),
				   GTK_OBJECT (window));
	gtk_container_add (GTK_CONTAINER (plug), eventbox);

	frame = gtk_frame_new (NULL);
	gtk_container_add (GTK_CONTAINER (eventbox), frame);

	vbox = gtk_vbox_new (FALSE, 5);
	gtk_container_add (GTK_CONTAINER (frame), vbox);

	if (gnome_config_get_bool ("/firestarter/Program/shutdown=TRUE")) {
		panelpic = gnome_pixmap_new_from_xpm_d (grey1_xpm);
		running = FALSE;
	}
	else {
		panelpic = gnome_pixmap_new_from_xpm_d (blue1_xpm);
		running = TRUE;
	}

	gtk_box_pack_start (GTK_BOX (vbox), panelpic, FALSE, FALSE, 2);

	install_docklet_menu (eventbox);

	gtk_widget_show_all (plug);
}

/*---[ pack_clist ]---------------------------------------------------
 * Pack a clist into a scrolled window and box
 *--------------------------------------------------------------------*/
static void
pack_clist (GtkWidget *clist, GtkWidget *box)
{
	GtkWidget *scrolled_win = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_win),
					GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

	gtk_container_add (GTK_CONTAINER (scrolled_win), clist);

	gtk_box_pack_start (GTK_BOX (box), scrolled_win, TRUE, TRUE, 0);
}

/*---[ prepare_clist ]------------------------------------------------
 * Convinience function that installs popups, signals and packs the clists
 *--------------------------------------------------------------------*/
void
prepare_clist (GtkWidget *clist, GtkWidget *vbox)
{
	install_clist_popup (clist);

	gtk_signal_connect (GTK_OBJECT (clist), "select-row",
			    GTK_SIGNAL_FUNC (clist_row_select), NULL);
	gtk_signal_connect (GTK_OBJECT (clist), "unselect-row",
			    GTK_SIGNAL_FUNC (clist_row_unselect), NULL);

	if (clist != hitlogclist)
		pack_clist (clist, vbox);
}

/*---[ exit_firestarter ]---------------------------------------------
 * Quit firestater
 *--------------------------------------------------------------------*/
void
exit_firestarter (void)
{
	if (gnome_config_get_bool ("/firestarter/Program/shutdown=TRUE"))
		stop_firewall ();
	gtk_main_quit ();
}

/*---[ closewindow ]--------------------------------------------------
 * Quit or hide, determined by the config files
 *--------------------------------------------------------------------*/
void
closewindow (void)
{
	if (gnome_config_get_bool ("/firestarter/Program/exitonclose=FALSE"))
		exit_firestarter ();
	else
		gtk_widget_hide (window);
}

/*---[ main ]---------------------------------------------------------
 * The main function, this is where it all begins
 *--------------------------------------------------------------------*/
int
main (int argc, char* argv[])
{
	GtkObject *status_dock;

	GtkWidget *notebook;
	GtkWidget *scrolled_win;
	GtkWidget *tablabel;

	gchar *hitlogtitles[4] = {_("Port"), _("Sent from"), _("Service"), _("Time")};

	GtkWidget *dynamicvbox;
	gchar *denytitles[1] = {_("Deny all connections from")};
	gchar *allowtitles[1] = {_("Allow all connections from")};
	gchar *allowsmtitles[1] = {_("Open service to machine")};
	gchar *allowsatitles[1] = {_("Open service to anyone")};

	gchar *logpath;
	FILE *file;
	CORBA_Environment ev;
	CORBA_ORB orb;
/*
	bindtextdomain ("firestarter", "/usr/share/locale/");
	textdomain ("firestarter");
*/
	CORBA_exception_init (&ev);
	orb = gnome_CORBA_init (PACKAGE, VERSION, &argc, argv, 0, &ev);

	window = gnome_app_new (PACKAGE, "Firestarter");

/* Set up the docklet */
	status_dock = status_docklet_new ();
	gtk_signal_connect (GTK_OBJECT (status_dock), "build_plug",
			    GTK_SIGNAL_FUNC (status_docklet_build_plug), NULL);
	status_docklet_run (STATUS_DOCKLET (status_dock));

/* Set up the window */
	gtk_signal_connect (GTK_OBJECT (window), "delete_event",
			    GTK_SIGNAL_FUNC (closewindow), NULL);

	gnome_window_icon_set_from_file (GTK_WINDOW (window), "/usr/share/pixmaps/firestarter.png");

	install_menus_and_toolbar (window);

	notebook = gtk_notebook_new ();
	gnome_app_set_contents (GNOME_APP (window), notebook);

/* set up hitlogclist */
	scrolled_win = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_win),
					GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);

	tablabel = gtk_label_new _("Firewall hits");
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), scrolled_win, tablabel);

	hitlogclist = gtk_clist_new_with_titles (4, hitlogtitles);
	prepare_clist (hitlogclist, NULL);
	gtk_container_add (GTK_CONTAINER (scrolled_win), hitlogclist);

/* set up the clists on the dynamic rules page */
	tablabel = gtk_label_new _("Dynamic rules");
	dynamicvbox = gtk_vbox_new (FALSE, 0);
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), dynamicvbox, tablabel);

	denyallclist = gtk_clist_new_with_titles (1, denytitles);
	gtk_object_set_data (GTK_OBJECT (denyallclist), "path", "/usr/local/firestarter/deny-all");
	prepare_clist (denyallclist, dynamicvbox);

	allowallclist = gtk_clist_new_with_titles (1, allowtitles);
	gtk_object_set_data (GTK_OBJECT (allowallclist), "path", "/usr/local/firestarter/allow-all");
	prepare_clist (allowallclist, dynamicvbox);

	allowsmclist = gtk_clist_new_with_titles (1, allowsmtitles);
	gtk_object_set_data (GTK_OBJECT (allowsmclist), "path", "/usr/local/firestarter/allow-service-machine");
	prepare_clist (allowsmclist, dynamicvbox);

	allowsaclist = gtk_clist_new_with_titles (1, allowsatitles);
	gtk_object_set_data (GTK_OBJECT (allowsaclist), "path", "/usr/local/firestarter/allow-service-all");
	prepare_clist (allowsaclist, dynamicvbox);

	logpath = gnome_config_get_string ("/firestarter/Files/logfile=/var/log/messages");

	/* Check if user has read access to the log file */
	if (fopen (logpath, "r") == NULL) {
		gchar *string;
		GtkWidget *dialog;
	
		string = g_strjoin (NULL, N_("Logfile \""), logpath,
				    N_("\" not found or access denied.\nFirewall log monitoring disabled.\n\n"
				       "You can configure the path to the log file in the "
				       "\"preferences\" section."),  NULL);
		dialog = gnome_error_dialog (string);
		gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
		logenabled = FALSE;
		gtk_widget_set_sensitive (GTK_WIDGET (hitlogclist), FALSE);

		g_free (string);
	}
	else {
		gchar buf[512];

		file = fopen (logpath, "r");
		/* Move to the end of the file */
		while (fgets (buf, 512, file) != NULL) continue;
		gtk_timeout_add (500, poll_log_timeout, file);
		g_free (buf);
	}

	mkdir ("/usr/local/firestarter", 00700);

	/* populate the clists on the dynamic rules notebookpage*/
	parse_deny_all ();
	parse_allow_all ();
	parse_allow_service_machine ();
	parse_allow_service_all ();

	if (gnome_config_get_bool ("/firestarter/Program/startup=FALSE"))
		restart_firewall ();

	appbar_update ();

	gtk_window_set_default_size (GTK_WINDOW (window), 300, 300);

	if (!gnome_config_get_bool ("/firestarter/Program/starthidden=FALSE"))
		gtk_widget_show_all (window);

	if (gnome_config_get_bool ("/firestarter/Druid/firsttime=TRUE"))
		rundruid ();

	gtk_main ();

	CORBA_exception_free (&ev);

	return 0;
}
