/*---[ parse.c ]------------------------------------------------------
 * Copyright (C) 2000 Tomas Junnonen (majix@sci.fi)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * The text file parse functions. Cut'n'Paste spaghetti land.
 *--------------------------------------------------------------------*/

#include "firestarter.h"
#include "parse.h"
#include "service.h"

/*---[ parse_hit_log ]------------------------------------------------
 * Parses *line. Extracts port, remoteip, service, time and protocol
 *--------------------------------------------------------------------*/
int
parse_hit_log (char *line)
{
	char *key = "DENY";
	gchar *loc1;
	gchar *loc2;
	gchar *data[4] = {"port", "remoteip", "service", "time"};
	gchar *proto;
	gint i;

	static gchar *time;
	static gchar *port;
	static gchar *ip;

	data[3] = g_strndup (line, 12);
	loc1 = (char *)strstr (line, key);

	if (loc1 != NULL) {

		/* Determine the protocol used, not in use at the moment */
		if (strstr (loc1, "PROTO=") != NULL)
			loc1 = (char *)strstr (loc1, "PROTO=");
		else
			return FALSE;
		loc1 += 6*(sizeof (char));
		proto = g_strndup (loc1, 2);
		/* do an atoi to remove possible trailing garbage */
		proto = g_strdup_printf ("%d", atoi (proto));

		/* Determine the senders IP */
		loc1 += 2*(sizeof (char));
		if (strchr (loc1, ':') != NULL)
			loc2 = (char *)strchr (loc1, ':');
		else
			return FALSE;
		data[1] = g_strndup (loc1, loc2-loc1);
		g_strstrip (data[1]);

		/* Determine the port the packet was sent to */
		if (strrchr (loc1, ':') != NULL)
			loc1 = (char *)strrchr (loc1, ':');
		else
			return FALSE;
		loc1 += (sizeof (char));
		data[0] = g_strndup (loc1, 5);
		data[0] = g_strdup_printf ("%d", atoi (data[0]));

		/* Determine the service used, based on used port */
		data[2] = service (atoi (data[0]));


		/* Remove identical lines */
		if (gnome_config_get_bool ("/firestarter/Program/removedups=FALSE")) {

			if (time != NULL) {
				if (!g_strcasecmp (time, data[3]))
					if (!g_strcasecmp (port, data[0]))
						if (!g_strcasecmp (ip, data[1]))
							return FALSE;
			}

			time = data[3];
			port = data[0];
			ip = data[1];
		}

		gtk_clist_append (GTK_CLIST (hitlogclist), data);

		/* Set the optimal width of the hitlogclist*/
		for (i = 0; i < 4; i++)
			gtk_clist_set_column_width (GTK_CLIST (hitlogclist), i,
						    gtk_clist_optimal_column_width (GTK_CLIST (hitlogclist), i));

		return TRUE;
	}
	return FALSE;
}

/*---[ parse_deny_all ]-----------------------------------------------
 * Parses the deny-all file, populates denyallclist
 *--------------------------------------------------------------------*/
void
parse_deny_all (void)
{
	char *key = "-j DENY";
	gchar *data[1];
	char buf[512];
	gchar *pos;

	FILE *f;

	if (fopen ("/usr/local/firestarter/deny-all", "r") == NULL)
		return;
	else
		f = fopen ("/usr/local/firestarter/deny-all", "r");

	gtk_clist_clear (GTK_CLIST (denyallclist));

	while (fgets (buf, 512, f) != NULL) {
		if (strstr (buf, key) != NULL)
			if (strstr (buf, "-s ") != NULL) {
				data[0] = (char *)strstr (buf, "-s ");
				data[0] += 3;
				pos = (char *)strstr (buf, "-d ");
				data[0] = g_strndup (data[0], pos-data[0]);
				g_strstrip (data[0]);
				gtk_clist_append (GTK_CLIST (denyallclist), data);
			}
	}
}

/*---[ parse_allow_all ]----------------------------------------------
 * Parses the allow-all file, populates allowallclist
 *--------------------------------------------------------------------*/
void
parse_allow_all (void)
{
	char *key = "-j ACCEPT";
	gchar *data[1];
	char buf[512];
	gchar *pos;

	FILE *f;

	if (fopen ("/usr/local/firestarter/allow-all", "r") == NULL)
		return;
	else
		f = fopen ("/usr/local/firestarter/allow-all", "r");

	gtk_clist_clear (GTK_CLIST (allowallclist));

	while (fgets (buf, 512, f) != NULL) {
		if (strstr (buf, key) != NULL)
			if (strstr (buf, "-s ") != NULL) {
				data[0] = (char *)strstr (buf, "-s ");
				data[0] += 3;
				pos = (char *)strstr (buf, "-d ");
				data[0] = g_strndup (data[0], pos-data[0]);
				g_strstrip (data[0]);
				gtk_clist_append (GTK_CLIST (allowallclist), data);
			}
	}
}

/*---[ parse_allow_service_machine ]----------------------------------
 * Parses the allow-service-machine file, populates allowsmclist
 *--------------------------------------------------------------------*/
void
parse_allow_service_machine (void)
{
	char *key = "-j ACCEPT";
	gchar *data[3];
	char buf[512];
	gchar *pos;
	gchar *entry[1];

	FILE *f;

	if (fopen ("/usr/local/firestarter/allow-service-machine", "r") == NULL)
		return;
	else
		f = fopen ("/usr/local/firestarter/allow-service-machine", "r");

	gtk_clist_clear (GTK_CLIST (allowsmclist));

	while (fgets (buf, 512, f) != NULL) {
		if (strstr (buf, key) != NULL)
			if (strstr (buf, "-s ") != NULL) {
				data[2] = (char *)strstr (buf, "-s ");
				data[2] += 3;
				pos = (char *)strstr (buf, "-d ");
				data[2] = g_strndup (data[2], pos-data[2]);
				g_strstrip (data[2]);

				data[1] = (char *)strstr (buf, "$NET ");
				data[1] += 4;
				pos = (char *)strstr (buf, "-j ");
				data[1] = g_strndup (data[1], pos-data[1]);
				g_strstrip (data[1]);

				data[0] = service (atoi (data[1]));

				entry[0] = g_strjoin (NULL, data[0],"/", data[1],"    ", data[2], NULL);
				gtk_clist_append (GTK_CLIST (allowsmclist), entry);
			}
	}
}

/*---[ parse_allow_service_all ]--------------------------------------
 * Parses the allow-service-all file, populates allowsaclist
 *--------------------------------------------------------------------*/
void
parse_allow_service_all (void)
{
	char *key = "-j ACCEPT";
	gchar *data[2];
	char buf[512];
	gchar *pos;
	gchar *entry[1];

	FILE *f;

	if (fopen ("/usr/local/firestarter/allow-service-all", "r") == NULL)
		return;
	else
		f = fopen ("/usr/local/firestarter/allow-service-all", "r");

	gtk_clist_clear (GTK_CLIST (allowsaclist));

	while (fgets (buf, 512, f) != NULL) {
		if (strstr (buf, key) != NULL)
			if (strstr (buf, "$NET ") != NULL) {
				data[1] = (char *)strstr (buf, "$NET ");
				data[1] += 4;
				pos = (char *)strstr (buf, "-j ");
				data[1] = g_strndup (data[1], pos-data[1]);
				g_strstrip (data[1]);

				data[0] = service (atoi (data[1]));

				entry[0] = g_strjoin (NULL, data[0], "/", data[1], NULL);
				gtk_clist_append (GTK_CLIST (allowsaclist), entry);
			}
	}
}
