/*---[ preferences.c ]------------------------------------------------
 * Copyright (C) 2000 Tomas Junnonen (majix@sci.fi)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * The preferences notebook window
 *--------------------------------------------------------------------*/

#include "preferences.h"

GtkWidget *scriptentry;
GtkWidget *logentry;

GtkWidget *startup;
GtkWidget *stopatexit;
GtkWidget *exitonclose;
GtkWidget *starthidden;
GtkWidget *removedups;

/*---[ starthidden_sensitive ]----------------------------------------
 * The sensitivity of the starthidden widget is determined from exitonclose
 *--------------------------------------------------------------------*/
static void
starthidden_sensitive (GtkWidget *widget)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (exitonclose)))
		gtk_widget_set_sensitive (starthidden, FALSE);
	else
		gtk_widget_set_sensitive (starthidden, TRUE);
}


/*---[ save_preferences.c ]-------------------------------------------
 * Save the preferences to the gnome registry
 *--------------------------------------------------------------------*/
static void
save_preferences (GnomePropertyBox *property_box, gint page_num, gint num)
{
	gnome_config_set_string ("/firestarter/Files/logfile",
				 gtk_entry_get_text (GTK_ENTRY (logentry)));

	gnome_config_set_string ("/firestarter/Files/scriptfile",
				 gtk_entry_get_text (GTK_ENTRY (scriptentry)));

	gnome_config_set_bool ("/firestarter/Program/startup",
			       gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (startup)));

	gnome_config_set_bool ("/firestarter/Program/shutdown",
			       gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (stopatexit)));

	gnome_config_set_bool ("/firestarter/Program/exitonclose",
			       gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (exitonclose)));

	if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (exitonclose)))
		gnome_config_set_bool ("/firestarter/Program/starthidden",
				       gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (starthidden)));
	else
		gnome_config_set_bool ("/firestarter/Program/starthidden", FALSE);

	gnome_config_set_bool ("/firestarter/Program/removedups",
			       gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (removedups)));


	gnome_config_sync ();
}

/*---[ preferences_cb ]-----------------------------------------------
 * Create the preferences property_box
 *--------------------------------------------------------------------*/
void 
preferences_cb (GtkWidget *widget, gpointer data)
{
	static GtkWidget *pbox;
	GtkWidget *page;
	GtkWidget *tab;
	GtkWidget *table;
	GtkWidget *label;

	/* Check that the window is not already created. Do not fix indentation */
	if (pbox != NULL) {
		g_assert (GTK_WIDGET_REALIZED (pbox));
		gdk_window_show (pbox->window);
		gdk_window_raise (pbox->window);
	}
	else {

	tab = gtk_label_new _("General");
	page = gtk_vbox_new (FALSE, 0);

	table = gtk_table_new (3, 2, FALSE);
	gtk_box_pack_start (GTK_BOX (page), table, FALSE, FALSE, 0);
	gtk_table_set_row_spacings (GTK_TABLE(table), 0);
	gtk_table_set_col_spacings (GTK_TABLE(table), GNOME_PAD_SMALL);
                                 
	label = gtk_label_new _("Firewall syslog file:");
	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.0);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
			  GTK_FILL, GTK_FILL, GNOME_PAD, GNOME_PAD_SMALL);

	logentry = gtk_entry_new ();
	gtk_widget_set_usize (logentry, 250, 20);
	gtk_entry_set_text (GTK_ENTRY (logentry),
			    gnome_config_get_string ("/firestarter/Files/logfile=/var/log/messages"));

	gtk_table_attach (GTK_TABLE (table), logentry, 1, 2, 0, 1,
			  GTK_FILL, GTK_FILL, GNOME_PAD, GNOME_PAD_SMALL);

	label = gtk_label_new _("Firewall script file:");
	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.0);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
			  GTK_FILL, GTK_FILL, GNOME_PAD, GNOME_PAD_SMALL);

	scriptentry = gtk_entry_new ();
	gtk_entry_set_text (GTK_ENTRY (scriptentry),
			    gnome_config_get_string ("/firestarter/Files/scriptfile=/usr/local/firestarter/firewall.sh"));
	gtk_table_attach (GTK_TABLE (table), scriptentry, 1, 2, 1, 2,
			  GTK_FILL, GTK_FILL, GNOME_PAD, GNOME_PAD_SMALL);

	startup = gtk_check_button_new_with_label _("Start firewall on program startup");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (startup), gnome_config_get_bool ("/firestarter/Program/startup=FALSE"));
	gtk_box_pack_start (GTK_BOX (page), startup, FALSE, FALSE, 1);

	stopatexit = gtk_check_button_new_with_label _("Stop firewall on program exit");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (stopatexit), gnome_config_get_bool ("/firestarter/Program/shutdown=TRUE"));
	gtk_box_pack_start (GTK_BOX (page), stopatexit, FALSE, FALSE, 1);

	exitonclose = gtk_check_button_new_with_label _("Exit program when window is closed (otherwise hide)");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (exitonclose), gnome_config_get_bool ("/firestarter/Program/exitonclose=FALSE"));
	gtk_box_pack_start (GTK_BOX (page), exitonclose, FALSE, FALSE, 1);

	starthidden = gtk_check_button_new_with_label _("Start hidden");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (starthidden), gnome_config_get_bool ("/firestarter/Program/starthidden=FALSE"));
	gtk_box_pack_start (GTK_BOX (page), starthidden, FALSE, FALSE, 0);

	removedups = gtk_check_button_new_with_label _("Remove identical firewall hit lines");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (removedups), gnome_config_get_bool ("/firestarter/Program/removedups=FALSE"));
	gtk_box_pack_start (GTK_BOX (page), removedups, FALSE, FALSE, 0);


	if (gnome_config_get_bool ("/firestarter/Program/exitonclose=FALSE"))
		gtk_widget_set_sensitive (starthidden, FALSE);

	pbox = gnome_property_box_new ();
	gnome_property_box_append_page (GNOME_PROPERTY_BOX (pbox), page, tab);

	gtk_signal_connect_object (GTK_OBJECT (logentry), "changed",
				   GTK_SIGNAL_FUNC (gnome_property_box_changed), GTK_OBJECT (pbox)); 
	gtk_signal_connect_object (GTK_OBJECT (scriptentry), "changed",
				   GTK_SIGNAL_FUNC (gnome_property_box_changed), GTK_OBJECT (pbox)); 
	gtk_signal_connect_object (GTK_OBJECT (startup), "toggled",
				   GTK_SIGNAL_FUNC (gnome_property_box_changed), GTK_OBJECT (pbox));
	gtk_signal_connect_object (GTK_OBJECT (stopatexit), "toggled",
				   GTK_SIGNAL_FUNC (gnome_property_box_changed), GTK_OBJECT (pbox));
	gtk_signal_connect_object (GTK_OBJECT (exitonclose), "toggled",
				   GTK_SIGNAL_FUNC (gnome_property_box_changed), GTK_OBJECT (pbox));
	gtk_signal_connect_object (GTK_OBJECT (removedups), "toggled",
				   GTK_SIGNAL_FUNC (gnome_property_box_changed), GTK_OBJECT (pbox));


	gtk_signal_connect (GTK_OBJECT (exitonclose), "toggled",
		            GTK_SIGNAL_FUNC (starthidden_sensitive), NULL);


	gtk_signal_connect_object (GTK_OBJECT (starthidden), "toggled",
				   GTK_SIGNAL_FUNC (gnome_property_box_changed), GTK_OBJECT (pbox));

	gtk_signal_connect (GTK_OBJECT (pbox), "apply",
			    GTK_SIGNAL_FUNC (save_preferences), NULL);

	gtk_signal_connect (GTK_OBJECT (pbox), "destroy",
			    GTK_SIGNAL_FUNC (gtk_widget_destroyed), &pbox);

	gtk_widget_show_all (pbox);
	}
}
