/*---[ scriptwriter.c ]-----------------------------------------------
 * Copyright (C) 2000 Tomas Junnonen (majix@sci.fi)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Creates the firewall script, based on wizard selections
 *--------------------------------------------------------------------*/

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "firestarter.h"
#include "druid.h"
#include "scriptwriter.h"


/*---[ check_file ]---------------------------------------------------
 * Check that file exists, if not, create
 *--------------------------------------------------------------------*/
static void
check_file (gchar *path)
{
	if (fopen (path, "r") == NULL) {
		FILE *file = fopen (path, "w");
		chmod (path, 00700);

		fprintf (file, "#!/bin/sh\n");

		fclose (file);
	}
}

/*---[ create_modrules_files ]----------------------------------------
 * Create the empty modrules scripts, unless already exists.
 *--------------------------------------------------------------------*/
static void
create_modrules_files (void)
{
	gchar *allow_allpath = "/usr/local/firestarter/allow-all";
	gchar *deny_allpath = "/usr/local/firestarter/deny-all";
	gchar *allow_service_machinepath = "/usr/local/firestarter/allow-service-machine";
	gchar *allow_service_allpath = "/usr/local/firestarter/allow-service-all";

	check_file (allow_allpath);
	check_file (deny_allpath);
	check_file (allow_service_machinepath);
	check_file (allow_service_allpath);
}

/*---[ druid_finished ]-----------------------------------------------
 * Creates the shell script, called when the finish button is pressed
 *--------------------------------------------------------------------*/
void
druid_finished (GnomeDruidPage *druidpage, gpointer druid, Druid *data)
{
	gchar *scriptpath = gnome_config_get_string ("/firestarter/Files/scriptfile=/usr/local/firestarter/firewall.sh");

	FILE *script = fopen (scriptpath, "w");

	chmod (scriptpath, 00700);

	fprintf (script, "#Generated by Firestarter\n");
	fprintf (script, "#!/bin/sh\n\n");
/* TODO - detect ipchains location */
	fprintf (script, "IPC=/sbin/ipchains\n");
/* The network interface */
	fprintf (script, "IF=%s\n", gtk_entry_get_text (GTK_ENTRY (data->device)));
/* IP address of interface */
	fprintf (script, "IP=`/sbin/ifconfig $IF | grep inet | cut -d : -f 2 | cut -d \\  -f 1`\n");
/* Netmask of interface */
	fprintf (script, "MASK=`/sbin/ifconfig $IF | grep Mas | cut -d : -f 4`\n");
	fprintf (script, "NET=$IP/$MASK\n\n");

	fprintf (script, "#Delete user made chains. Flush and zero the chains.\n");
	fprintf (script, "$IPC -X\n$IPC -F\n$IPC -Z\n\n");

/* Pipe in the modrules files */
	fprintf (script, "#Deny all traffic from these machines\n");
	fprintf (script, ". /usr/local/firestarter/deny-all\n");
	fprintf (script, "#Allow all traffic from these machines\n");
	fprintf (script, ". /usr/local/firestarter/allow-all\n");
	fprintf (script, "#Allow a specific service to a specific machine\n");
	fprintf (script, ". /usr/local/firestarter/allow-service-machine\n");
	fprintf (script, "#Allow a specific service to everyone\n");
	fprintf (script, ". /usr/local/firestarter/allow-service-all\n\n");

/* GOOD IDEA(tm) RULES */

	fprintf (script, "#Allow all traffic on the loopback interface\n");
	fprintf (script, "$IPC -A input -i lo -s 0/0 -d 0/0 -j ACCEPT\n");
	fprintf (script, "$IPC -A output -i lo -s 0/0 -d 0/0 -j ACCEPT\n\n");

	fprintf (script, "#Allow connections with the ack bit set\n");	
	fprintf (script, "$IPC -A input ! -y -p tcp -s 0/0 -d $NET -j ACCEPT\n\n");

	fprintf (script, "#Turn on source address verification in kernel\n");	
	fprintf (script, "if [ -e /proc/sys/net/ipv4/conf/all/rp_filter ]\n then\n"
	"  for f in /proc/sys/net/ipv4/conf/*/rp_filter\n  do\n   echo 2 > $f\n  done\nfi\n\n");

	fprintf (script, "#Turn on syn cookies protection in kernel\n");	
	fprintf (script, "if [ -e /proc/sys/net/ipv4/tcp_syncookies ]\n then\n"
	"  echo 1 > /proc/sys/net/ipv4/tcp_syncookies\nfi\n\n");

	fprintf (script, "#Turn on dynamic TCP/IP address hacking\n");
	fprintf (script, "if [ -e /proc/sys/net/ipv4/ip_dynaddr ]\n then\n"
	"  echo 1 > /proc/sys/net/ipv4/ip_dynaddr\nfi\n\n");
	
	fprintf (script, "#Block nonroutable IPs\n");	
/* Possible problem for some people? */
/*	fprintf (script, "$IPC -A input -j DENY -s 10.0.0.0/8 -d $NET -i $IF\n"); */
	fprintf (script, "$IPC -A input -j DENY -s 127.0.0.0/8 -d $NET -i $IF\n");
	fprintf (script, "$IPC -A input -j DENY -s 172.16.0.0/12 -d $NET -i $IF\n");
	fprintf (script, "$IPC -A input -j DENY -s 192.168.0.0/16 -d $NET -i $IF\n\n");

	fprintf (script, "#Block Back Orifice\n");	
	fprintf (script, "$IPC -A input -p tcp -s 0/0 -d $NET 31337 -j DENY -l\n");
	fprintf (script, "$IPC -A input -p udp -s 0/0 -d $NET 31337 -j DENY -l\n\n");

	fprintf (script, "#Block NetBus\n");	
	fprintf (script, "$IPC -A input -p tcp -s 0/0 -d $NET 12345:12346 -j DENY -l\n");
	fprintf (script, "$IPC -A input -p udp -s 0/0 -d $NET 12345:12346 -j DENY -l\n\n");

	fprintf (script, "#Block Trin00\n");	
	fprintf (script, "$IPC -A input -p tcp -s 0/0 -d $NET 1524 -j DENY -l\n");
	fprintf (script, "$IPC -A input -p tcp -s 0/0 -d $NET 27665 -j DENY -l\n");
	fprintf (script, "$IPC -A input -p udp -s 0/0 -d $NET 27444 -j DENY -l\n");
	fprintf (script, "$IPC -A input -p udp -s 0/0 -d $NET 31335 -j DENY -l\n\n");

	fprintf (script, "#Block Multicast\n");	
	fprintf (script, "$IPC -A input -s 224.0.0.0/8 -d 0/0 -j DENY\n");
	fprintf (script, "$IPC -A input -s 0/0 -d 224.0.0.0/8 -j DENY\n\n");

/* RULES BASED ON WIZARD SELECTIONS */

if GTK_WIDGET_IS_SENSITIVE (data->services)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->ftp))) {
		fprintf (script, "#FTP\n");
		fprintf (script, "$IPC -A input -p tcp -s 0/0 -d $NET 20 -j ACCEPT\n");
		fprintf (script, "$IPC -A input -p tcp -s 0/0 -d $NET 21 -j ACCEPT\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->ssh))) {
		fprintf (script, "#SSH\n");		
		fprintf (script, "$IPC -A input -p tcp -s 0/0 -d $NET 22 -j ACCEPT\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->telnet))) {
		fprintf (script, "#Telnet\n");
		fprintf (script, "$IPC -A input -p tcp -s 0/0 -d $NET 23 -j ACCEPT\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->smtp))) {
		fprintf (script, "#SMTP\n");
		fprintf (script, "$IPC -A input -p tcp -s 0/0 -d $NET 25 -j ACCEPT\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->dns))) {
		fprintf (script, "#DNS\n");
		fprintf (script, "$IPC -A input -p tcp -s 0/0 -d $NET 53 -j ACCEPT\n");
		fprintf (script, "$IPC -A input -p udp -s 0/0 -d $NET 53 -j ACCEPT\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->finger))) {
		fprintf (script, "#Finger\n");
		fprintf (script, "$IPC -A input -p tcp -s 0/0 -d $NET 79 -j ACCEPT\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->www))) {
		fprintf (script, "#HTTP\n");
		fprintf (script, "$IPC -A input -p tcp -s 0/0 -d $NET 80 -j ACCEPT\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->sslweb))) {
		fprintf (script, "#SSL HTTP\n");
		fprintf (script, "$IPC -A input -p tcp -s 0/0 -d $NET 443 -j ACCEPT\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->pop))) {
		fprintf (script, "#POP\n");
		fprintf (script, "$IPC -A input -p tcp -s 0/0 -d $NET 110 -j ACCEPT\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->imap))) {
		fprintf (script, "#IMAP\n");
		fprintf (script, "$IPC -A input -p tcp -s 0/0 -d $NET 143 -j ACCEPT\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->ident))) {
		fprintf (script, "#IDENT\n");
		fprintf (script, "$IPC -A input -p tcp -s 0/0 -d $NET 113 -j ACCEPT\n");
		fprintf (script, "$IPC -A input -p udp -s 0/0 -d $NET 113 -j ACCEPT\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->nntp))) {
		fprintf (script, "#NNTP\n");
		fprintf (script, "$IPC -A input -p tcp -s 0/0 -d $NET 119 -j ACCEPT\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->ntp))) {
		fprintf (script, "#NTP\n");		
		fprintf (script, "$IPC -A input -p tcp -s 0/0 -d $NET 123 -j ACCEPT\n");
		fprintf (script, "$IPC -A input -p udp -s 0/0 -d $NET 123 -j ACCEPT\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->samba))) {
		fprintf (script, "#SAMBA\n");
		fprintf (script, "$IPC -A input -p tcp -s 0/0 -d 0/0 137:139 -i $IF -j ACCEPT\n");
		fprintf (script, "$IPC -A input -p udp -s 0/0 -d 0/0 137:139 -i $IF -j ACCEPT\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->routed))) {
		fprintf (script, "#Routed\n");
		fprintf (script, "$IPC -A input -p udp -s 0/0 -d 0/0 520 -j ACCEPT\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->nfs))) {
		fprintf (script, "#NFS\n");
		fprintf (script, "$IPC -A input -p tcp -s 0/0 -d 0/0 2049 -i $IF -j ACCEPT\n");
		fprintf (script, "$IPC -A input -p udp -s 0/0 -d 0/0 2049 -i $IF -j ACCEPT\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->x))) {
		fprintf (script, "#Xwindows\n");
		fprintf (script, "$IPC -A input -p tcp -s 0/0 -d 0/0 6000:6010 -i $IF -j ACCEPT\n");
		fprintf (script, "$IPC -A input -p udp -s 0/0 -d 0/0 6000:6010 -i $IF -j ACCEPT\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->dhcp))) {
		fprintf (script, "#DHCP\n");
		fprintf (script, "$IPC -A input -p udp -s 0/0 -d 0/0 67:68 -i $IF -j ACCEPT\n\n");
	}
}

/* REQUIRED RULES */

	fprintf (script, "#Allow ICMP\n");
	fprintf (script, "$IPC -A input -p icmp -s 0/0 -d $NET -j ACCEPT\n");
	fprintf (script, "$IPC -A output -p icmp -s $NET -d 0/0 -j ACCEPT\n\n");

	fprintf (script, "#Open ports for established connections\n");
	fprintf (script, "$IPC -A input -p tcp -s 0/0 -d $NET 1023:49151 -j ACCEPT\n");
	fprintf (script, "$IPC -A input -p udp -s 0/0 -d $NET 1023:49151 -j ACCEPT\n\n");
/*
	fprintf (script, "$IPC -A input -p tcp -s 0/0 -d $NET 1023:65535 -j ACCEPT\n");
	fprintf (script, "$IPC -A input -p udp -s 0/0 -d $NET 1023:65535 -j ACCEPT\n\n");
*/
	fprintf (script, "#Deny everything not let through earlier\n");	
	fprintf (script, "$IPC -A output -j ACCEPT\n");
	fprintf (script, "$IPC -A input -j DENY -l\n");

	fclose (script);

	create_modrules_files ();

	restart_firewall ();

	g_print (N_("Firewall started, firewall script saves as %s\n"), scriptpath);
	quit_druid ();
}
